@@ -1,16 +1,18 @@
-# Copyright (c) 2019-2021, Jonas Eschle, Jim Pivarski, Eduardo Rodrigues, and Henry Schreiner.
+# Copyright (c) 2019-2024, Jonas Eschle, Jim Pivarski, Eduardo Rodrigues, and Henry Schreiner.
 #
 # Distributed under the 3-clause BSD license, see accompanying file LICENSE
 # or https://github.com/scikit-hep/vector for details.
 
-import typing
-
 """
 .. code-block:: python
 
     Lorentz.boostX(self, gamma=)
 """
 
+from __future__ import annotations
+
+import typing
+
 import numpy
 
 from vector._compute.lorentz import t
@@ -246,7 +248,7 @@ def dispatch(gamma: typing.Any, v: typing.Any) -> typing.Any:
                 gamma,
                 *v.azimuthal.elements,
                 *v.longitudinal.elements,
-                *v.temporal.elements
+                *v.temporal.elements,
             ),
             returns,
             1,
