@@ -1,10 +1,8 @@
-# Copyright (c) 2019-2021, Jonas Eschle, Jim Pivarski, Eduardo Rodrigues, and Henry Schreiner.
+# Copyright (c) 2019-2024, Jonas Eschle, Jim Pivarski, Eduardo Rodrigues, and Henry Schreiner.
 #
 # Distributed under the 3-clause BSD license, see accompanying file LICENSE
 # or https://github.com/scikit-hep/vector for details.
 
-import typing
-
 """
 .. code-block:: python
 
@@ -17,6 +15,10 @@ or
     Lorentz.boost(self, beta3=...)
 """
 
+from __future__ import annotations
+
+import typing
+
 import numpy
 
 from vector._compute.lorentz import transform4D
@@ -208,14 +210,24 @@ dispatch_map = {
         TemporalT,
         AzimuthalXY,
         LongitudinalTheta,
-    ): (cartesian_t_xy_theta, AzimuthalXY, LongitudinalZ, TemporalT),
+    ): (
+        cartesian_t_xy_theta,
+        AzimuthalXY,
+        LongitudinalZ,
+        TemporalT,
+    ),
     (
         AzimuthalXY,
         LongitudinalZ,
         TemporalTau,
         AzimuthalXY,
         LongitudinalTheta,
-    ): (cartesian_tau_xy_theta, AzimuthalXY, LongitudinalZ, TemporalTau),
+    ): (
+        cartesian_tau_xy_theta,
+        AzimuthalXY,
+        LongitudinalZ,
+        TemporalTau,
+    ),
     (AzimuthalXY, LongitudinalZ, TemporalT, AzimuthalXY, LongitudinalEta): (
         cartesian_t_xy_eta,
         AzimuthalXY,
@@ -234,42 +246,72 @@ dispatch_map = {
         TemporalT,
         AzimuthalRhoPhi,
         LongitudinalZ,
-    ): (cartesian_t_rhophi_z, AzimuthalXY, LongitudinalZ, TemporalT),
+    ): (
+        cartesian_t_rhophi_z,
+        AzimuthalXY,
+        LongitudinalZ,
+        TemporalT,
+    ),
     (
         AzimuthalXY,
         LongitudinalZ,
         TemporalTau,
         AzimuthalRhoPhi,
         LongitudinalZ,
-    ): (cartesian_tau_rhophi_z, AzimuthalXY, LongitudinalZ, TemporalTau),
+    ): (
+        cartesian_tau_rhophi_z,
+        AzimuthalXY,
+        LongitudinalZ,
+        TemporalTau,
+    ),
     (
         AzimuthalXY,
         LongitudinalZ,
         TemporalT,
         AzimuthalRhoPhi,
         LongitudinalTheta,
-    ): (cartesian_t_rhophi_theta, AzimuthalXY, LongitudinalZ, TemporalT),
+    ): (
+        cartesian_t_rhophi_theta,
+        AzimuthalXY,
+        LongitudinalZ,
+        TemporalT,
+    ),
     (
         AzimuthalXY,
         LongitudinalZ,
         TemporalTau,
         AzimuthalRhoPhi,
         LongitudinalTheta,
-    ): (cartesian_tau_rhophi_theta, AzimuthalXY, LongitudinalZ, TemporalTau),
+    ): (
+        cartesian_tau_rhophi_theta,
+        AzimuthalXY,
+        LongitudinalZ,
+        TemporalTau,
+    ),
     (
         AzimuthalXY,
         LongitudinalZ,
         TemporalT,
         AzimuthalRhoPhi,
         LongitudinalEta,
-    ): (cartesian_t_rhophi_eta, AzimuthalXY, LongitudinalZ, TemporalT),
+    ): (
+        cartesian_t_rhophi_eta,
+        AzimuthalXY,
+        LongitudinalZ,
+        TemporalT,
+    ),
     (
         AzimuthalXY,
         LongitudinalZ,
         TemporalTau,
         AzimuthalRhoPhi,
         LongitudinalEta,
-    ): (cartesian_tau_rhophi_eta, AzimuthalXY, LongitudinalZ, TemporalTau),
+    ): (
+        cartesian_tau_rhophi_eta,
+        AzimuthalXY,
+        LongitudinalZ,
+        TemporalTau,
+    ),
 }
 
 
