@@ -1,4 +1,4 @@
-# Copyright (c) 2019-2021, Jonas Eschle, Jim Pivarski, Eduardo Rodrigues, and Henry Schreiner.
+# Copyright (c) 2019-2024, Jonas Eschle, Jim Pivarski, Eduardo Rodrigues, and Henry Schreiner.
 #
 # Distributed under the 3-clause BSD license, see accompanying file LICENSE
 # or https://github.com/scikit-hep/vector for details.
@@ -9,6 +9,8 @@ Defines behaviors for Awkward Array. New arrays created with the
 .. code-block:: python
 
     vector.Array(...)
+    vector.awk(...)
+    vector.zip(...)
 
 function will have these behaviors built in (and will pass them to any derived
 arrays).
@@ -120,8 +122,7 @@ class AzimuthalAwkward(CoordinatesAwkward, Azimuthal):
             >>> az
             AzimuthalAwkwardXY(<Array [[1, 2], None] type='2 * option[var * int64]'>, <Array [None, [1]] type='2 * option[var * int64]'>)
             >>> az.elements
-            (<Array [[1, 2], None] type='2 * option[var * int64]'>, <Array [None, [1]] type='2 * option[var
-            * int64]'>)
+            (<Array [[1, 2], None] type='2 * option[var * int64]'>, <Array [None, [1]] type='2 * option[var * int64]'>)
         """
         fields = ak.fields(array)
         if "x" in fields and "y" in fields:
@@ -149,8 +150,7 @@ class AzimuthalAwkward(CoordinatesAwkward, Azimuthal):
             >>> az
             AzimuthalAwkwardXY(<Array [[1, 2], None] type='2 * option[var * int64]'>, <Array [None, [1]] type='2 * option[var * int64]'>)
             >>> az.elements
-            (<Array [[1, 2], None] type='2 * option[var * int64]'>, <Array [None, [1]] type='2 * option[var
-            * int64]'>)
+            (<Array [[1, 2], None] type='2 * option[var * int64]'>, <Array [None, [1]] type='2 * option[var * int64]'>)
         """
         fields = ak.fields(array)
         if "x" in fields and "y" in fields:
@@ -575,20 +575,21 @@ class TemporalAwkwardTau(TemporalAwkward, TemporalTau):
 
 
 def _class_to_name(cls: type[VectorProtocol]) -> str:
+    # respect the type of classes inheriting VectorAwkward classes
+    is_vector = "vector.backends" in cls.__module__
     if issubclass(cls, Momentum):
         if issubclass(cls, Vector2D):
-            return "Momentum2D"
-        elif issubclass(cls, Vector3D):
-            return "Momentum3D"
-        elif issubclass(cls, Vector4D):
-            return "Momentum4D"
-    else:
-        if issubclass(cls, Vector2D):
-            return "Vector2D"
-        elif issubclass(cls, Vector3D):
-            return "Vector3D"
-        elif issubclass(cls, Vector4D):
-            return "Vector4D"
+            return "Momentum2D" if is_vector else cls.__name__[:-5]
+        if issubclass(cls, Vector3D):
+            return "Momentum3D" if is_vector else cls.__name__[:-5]
+        if issubclass(cls, Vector4D):
+            return "Momentum4D" if is_vector else cls.__name__[:-5]
+    if issubclass(cls, Vector2D):
+        return "Vector2D" if is_vector else cls.__name__[:-5]
+    if issubclass(cls, Vector3D):
+        return "Vector3D" if is_vector else cls.__name__[:-5]
+    if issubclass(cls, Vector4D):
+        return "Vector4D" if is_vector else cls.__name__[:-5]
 
     raise AssertionError(repr(cls))
 
@@ -608,12 +609,11 @@ def _no_record(x: ak.Array) -> ak.Array | None:
 
 # Type for mixing in Awkward later
 class AwkwardProtocol(Protocol):
-    def __getitem__(self, where: typing.Any) -> float | ak.Array | ak.Record | None:
-        ...
+    def __getitem__(self, where: typing.Any) -> float | ak.Array | ak.Record | None: ...
 
 
 class VectorAwkward:
-    """One dimensional vector class for the Awkward backend."""
+    """Mixin class for Awkward vectors."""
 
     lib: types.ModuleType = numpy
 
@@ -655,7 +655,7 @@ class VectorAwkward:
             and isinstance(returns[0], type)
             and issubclass(returns[0], Azimuthal)
         ):
-            first = [x for x in result if isinstance(x, ak.Array)][0]
+            first = next(x for x in result if isinstance(x, ak.Array))
             result = [
                 x if isinstance(x, ak.Array) else ak.broadcast_arrays(first, x)[1]
                 for x in result
@@ -673,7 +673,13 @@ class VectorAwkward:
             fields = ak.fields(self)
             if num_vecargs == 1:
                 for name in fields:
-                    if name not in ("x", "y", "rho", "phi"):
+                    if name not in (
+                        "x",
+                        "y",
+                        "rho",
+                        "pt",
+                        "phi",
+                    ):
                         names.append(name)
                         arrays.append(self[name])
 
@@ -699,7 +705,7 @@ class VectorAwkward:
             and issubclass(returns[0], Azimuthal)
             and returns[1] is None
         ):
-            first = [x for x in result if isinstance(x, ak.Array)][0]
+            first = next(x for x in result if isinstance(x, ak.Array))
             result = [
                 x if isinstance(x, ak.Array) else ak.broadcast_arrays(first, x)[1]
                 for x in result
@@ -720,12 +726,20 @@ class VectorAwkward:
                         "x",
                         "y",
                         "rho",
+                        "pt",
                         "phi",
                         "z",
+                        "pz",
                         "theta",
                         "eta",
                         "t",
                         "tau",
+                        "m",
+                        "M",
+                        "mass",
+                        "e",
+                        "E",
+                        "energy",
                     ):
                         names.append(name)
                         arrays.append(self[name])
@@ -746,7 +760,7 @@ class VectorAwkward:
             and isinstance(returns[1], type)
             and issubclass(returns[1], Longitudinal)
         ):
-            first = [x for x in result if isinstance(x, ak.Array)][0]
+            first = next(x for x in result if isinstance(x, ak.Array))
             result = [
                 x if isinstance(x, ak.Array) else ak.broadcast_arrays(first, x)[1]
                 for x in result
@@ -774,7 +788,17 @@ class VectorAwkward:
             fields = ak.fields(self)
             if num_vecargs == 1:
                 for name in fields:
-                    if name not in ("x", "y", "rho", "phi", "z", "theta", "eta"):
+                    if name not in (
+                        "x",
+                        "y",
+                        "rho",
+                        "pt",
+                        "phi",
+                        "z",
+                        "pz",
+                        "theta",
+                        "eta",
+                    ):
                         names.append(name)
                         arrays.append(self[name])
 
@@ -800,7 +824,7 @@ class VectorAwkward:
             and issubclass(returns[1], Longitudinal)
             and returns[2] is None
         ):
-            first = [x for x in result if isinstance(x, ak.Array)][0]
+            first = next(x for x in result if isinstance(x, ak.Array))
             result = [
                 x if isinstance(x, ak.Array) else ak.broadcast_arrays(first, x)[1]
                 for x in result
@@ -831,12 +855,20 @@ class VectorAwkward:
                         "x",
                         "y",
                         "rho",
+                        "pt",
                         "phi",
                         "z",
+                        "pz",
                         "theta",
                         "eta",
                         "t",
                         "tau",
+                        "m",
+                        "M",
+                        "mass",
+                        "e",
+                        "E",
+                        "energy",
                     ):
                         names.append(name)
                         arrays.append(self[name])
@@ -859,7 +891,7 @@ class VectorAwkward:
             and isinstance(returns[2], type)
             and issubclass(returns[2], Temporal)
         ):
-            first = [x for x in result if isinstance(x, ak.Array)][0]
+            first = next(x for x in result if isinstance(x, ak.Array))
             result = [
                 x if isinstance(x, ak.Array) else ak.broadcast_arrays(first, x)[1]
                 for x in result
@@ -897,12 +929,20 @@ class VectorAwkward:
                         "x",
                         "y",
                         "rho",
+                        "pt",
                         "phi",
                         "z",
+                        "pz",
                         "theta",
                         "eta",
                         "t",
                         "tau",
+                        "m",
+                        "M",
+                        "mass",
+                        "e",
+                        "E",
+                        "energy",
                     ):
                         names.append(name)
                         arrays.append(self[name])
@@ -1252,8 +1292,6 @@ class VectorRecord2D(VectorAwkward2D, ak.Record):  # type: ignore[misc]
     for the corresponding ``Vector`` and ``Momentum`` classes.
     """
 
-    pass
-
 
 behavior["Vector2D"] = VectorRecord2D
 
@@ -1288,8 +1326,6 @@ class VectorRecord3D(VectorAwkward3D, ak.Record):  # type: ignore[misc]
     for the corresponding ``Vector`` and ``Momentum`` classes.
     """
 
-    pass
-
 
 behavior["Vector3D"] = VectorRecord3D
 
@@ -1324,8 +1360,6 @@ class VectorRecord4D(VectorAwkward4D, ak.Record):  # type: ignore[misc]
     for the corresponding ``Vector`` and ``Momentum`` classes.
     """
 
-    pass
-
 
 behavior["Vector4D"] = VectorRecord4D
 
@@ -1360,8 +1394,6 @@ class MomentumRecord2D(MomentumAwkward2D, ak.Record):  # type: ignore[misc]
     for the corresponding ``Momentum`` and ``Vector`` classes.
     """
 
-    pass
-
 
 behavior["Momentum2D"] = MomentumRecord2D
 
@@ -1395,8 +1427,6 @@ class MomentumRecord3D(MomentumAwkward3D, ak.Record):  # type: ignore[misc]
     for the corresponding ``Momentum`` and ``Vector`` classes.
     """
 
-    pass
-
 
 behavior["Momentum3D"] = MomentumRecord3D
 
@@ -1430,8 +1460,6 @@ class MomentumRecord4D(MomentumAwkward4D, ak.Record):  # type: ignore[misc]
     for the corresponding ``Momentum`` and ``Vector`` classes.
     """
 
-    pass
-
 
 behavior["Momentum4D"] = MomentumRecord4D
 
@@ -1539,61 +1567,73 @@ VectorArray2D.ProjectionClass2D = VectorArray2D
 VectorArray2D.ProjectionClass3D = VectorArray3D
 VectorArray2D.ProjectionClass4D = VectorArray4D
 VectorArray2D.GenericClass = VectorArray2D
+VectorArray2D.MomentumClass = MomentumArray2D
 
 VectorRecord2D.ProjectionClass2D = VectorRecord2D
 VectorRecord2D.ProjectionClass3D = VectorRecord3D
 VectorRecord2D.ProjectionClass4D = VectorRecord4D
 VectorRecord2D.GenericClass = VectorRecord2D
+VectorRecord2D.MomentumClass = MomentumRecord2D
 
 MomentumArray2D.ProjectionClass2D = MomentumArray2D
 MomentumArray2D.ProjectionClass3D = MomentumArray3D
 MomentumArray2D.ProjectionClass4D = MomentumArray4D
 MomentumArray2D.GenericClass = VectorArray2D
+MomentumArray2D.MomentumClass = MomentumArray2D
 
 MomentumRecord2D.ProjectionClass2D = MomentumRecord2D
 MomentumRecord2D.ProjectionClass3D = MomentumRecord3D
 MomentumRecord2D.ProjectionClass4D = MomentumRecord4D
 MomentumRecord2D.GenericClass = VectorRecord2D
+MomentumRecord2D.MomentumClass = MomentumRecord2D
 
 VectorArray3D.ProjectionClass2D = VectorArray2D
 VectorArray3D.ProjectionClass3D = VectorArray3D
 VectorArray3D.ProjectionClass4D = VectorArray4D
 VectorArray3D.GenericClass = VectorArray3D
+VectorArray3D.MomentumClass = MomentumArray3D
 
 VectorRecord3D.ProjectionClass2D = VectorRecord2D
 VectorRecord3D.ProjectionClass3D = VectorRecord3D
 VectorRecord3D.ProjectionClass4D = VectorRecord4D
 VectorRecord3D.GenericClass = VectorRecord3D
+VectorRecord3D.MomentumClass = MomentumRecord3D
 
 MomentumArray3D.ProjectionClass2D = MomentumArray2D
 MomentumArray3D.ProjectionClass3D = MomentumArray3D
 MomentumArray3D.ProjectionClass4D = MomentumArray4D
 MomentumArray3D.GenericClass = VectorArray3D
+MomentumArray3D.MomentumClass = MomentumArray3D
 
 MomentumRecord3D.ProjectionClass2D = MomentumRecord2D
 MomentumRecord3D.ProjectionClass3D = MomentumRecord3D
 MomentumRecord3D.ProjectionClass4D = MomentumRecord4D
 MomentumRecord3D.GenericClass = VectorRecord3D
+MomentumRecord3D.MomentumClass = MomentumRecord3D
 
 VectorArray4D.ProjectionClass2D = VectorArray2D
 VectorArray4D.ProjectionClass3D = VectorArray3D
 VectorArray4D.ProjectionClass4D = VectorArray4D
 VectorArray4D.GenericClass = VectorArray4D
+VectorArray4D.MomentumClass = MomentumArray4D
 
 VectorRecord4D.ProjectionClass2D = VectorRecord2D
 VectorRecord4D.ProjectionClass3D = VectorRecord3D
 VectorRecord4D.ProjectionClass4D = VectorRecord4D
 VectorRecord4D.GenericClass = VectorRecord4D
+VectorRecord4D.MomentumClass = MomentumRecord4D
 
 MomentumArray4D.ProjectionClass2D = MomentumArray2D
 MomentumArray4D.ProjectionClass3D = MomentumArray3D
 MomentumArray4D.ProjectionClass4D = MomentumArray4D
 MomentumArray4D.GenericClass = VectorArray4D
+MomentumArray4D.MomentumClass = MomentumArray4D
 
 MomentumRecord4D.ProjectionClass2D = MomentumRecord2D
 MomentumRecord4D.ProjectionClass3D = MomentumRecord3D
 MomentumRecord4D.ProjectionClass4D = MomentumRecord4D
 MomentumRecord4D.GenericClass = VectorRecord4D
+MomentumRecord4D.MomentumClass = MomentumRecord4D
 
 
 # implementation of behaviors in Numba ########################################
@@ -1627,7 +1667,7 @@ def _arraytype_of(awkwardtype: typing.Any, component: str) -> typing.Any:
     ):
         return _arraytype_of(awkwardtype.contenttype, component)
     raise numba.TypingError(
-        f"vector components like {repr(component)} must be NumpyArrayType, not {awkwardtype}"
+        f"vector components like {component!r} must be NumpyArrayType, not {awkwardtype}"
     )
 
 
@@ -1702,9 +1742,11 @@ def _aztype_of(recordarraytype: typing.Any, is_momentum: bool) -> typing.Any:
 def _ltype_of(recordarraytype: typing.Any, is_momentum: bool) -> typing.Any:
     import numba
 
-    cls: type[LongitudinalObjectZ] | type[LongitudinalObjectTheta] | type[
-        LongitudinalObjectEta
-    ]
+    cls: (
+        type[LongitudinalObjectZ]
+        | type[LongitudinalObjectTheta]
+        | type[LongitudinalObjectEta]
+    )
 
     z_index = None
     theta_index = None
@@ -1990,16 +2032,85 @@ def _numba_lower(
     return context.compile_internal(builder, impl, sig, args)
 
 
-ak.behavior["__numba_typer__", "Vector2D"] = _numba_typer_Vector2D
-ak.behavior["__numba_typer__", "Vector3D"] = _numba_typer_Vector3D
-ak.behavior["__numba_typer__", "Vector4D"] = _numba_typer_Vector4D
-ak.behavior["__numba_typer__", "Momentum2D"] = _numba_typer_Momentum2D
-ak.behavior["__numba_typer__", "Momentum3D"] = _numba_typer_Momentum3D
-ak.behavior["__numba_typer__", "Momentum4D"] = _numba_typer_Momentum4D
+behavior["__numba_typer__", "Vector2D"] = _numba_typer_Vector2D
+behavior["__numba_typer__", "Vector3D"] = _numba_typer_Vector3D
+behavior["__numba_typer__", "Vector4D"] = _numba_typer_Vector4D
+behavior["__numba_typer__", "Momentum2D"] = _numba_typer_Momentum2D
+behavior["__numba_typer__", "Momentum3D"] = _numba_typer_Momentum3D
+behavior["__numba_typer__", "Momentum4D"] = _numba_typer_Momentum4D
 
-ak.behavior["__numba_lower__", "Vector2D"] = _numba_lower
-ak.behavior["__numba_lower__", "Vector3D"] = _numba_lower
-ak.behavior["__numba_lower__", "Vector4D"] = _numba_lower
-ak.behavior["__numba_lower__", "Momentum2D"] = _numba_lower
-ak.behavior["__numba_lower__", "Momentum3D"] = _numba_lower
-ak.behavior["__numba_lower__", "Momentum4D"] = _numba_lower
+behavior["__numba_lower__", "Vector2D"] = _numba_lower
+behavior["__numba_lower__", "Vector3D"] = _numba_lower
+behavior["__numba_lower__", "Vector4D"] = _numba_lower
+behavior["__numba_lower__", "Momentum2D"] = _numba_lower
+behavior["__numba_lower__", "Momentum3D"] = _numba_lower
+behavior["__numba_lower__", "Momentum4D"] = _numba_lower
+
+
+def _reduce_sum(
+    array: VectorArray2D
+    | VectorArray3D
+    | VectorArray4D
+    | MomentumArray2D
+    | MomentumArray3D
+    | MomentumArray4D,
+    mask_identity: bool,
+) -> VectorProtocol:
+    fields = {}
+    if isinstance(array, Lorentz):
+        fields["t"] = numpy.sum(array.t, axis=1)
+    if isinstance(array, Spatial):
+        fields["z"] = numpy.sum(array.z, axis=1)
+
+    assert isinstance(array, Planar)
+    fields["x"] = numpy.sum(array.x, axis=1)
+    fields["y"] = numpy.sum(array.y, axis=1)
+
+    layout = ak.to_layout(array)
+
+    return ak.zip(
+        fields,
+        behavior=array.behavior,
+        with_name=layout.purelist_parameter("__record__"),
+    )
+
+
+def _reduce_count(
+    array: VectorArray2D
+    | VectorArray3D
+    | VectorArray4D
+    | MomentumArray2D
+    | MomentumArray3D
+    | MomentumArray4D,
+    mask_identity: bool,
+) -> VectorProtocol:
+    first_field = array[array.fields[0]]
+    return ak.count(first_field, axis=1)
+
+
+def _reduce_count_nonzero(
+    array: VectorArray2D
+    | VectorArray3D
+    | VectorArray4D
+    | MomentumArray2D
+    | MomentumArray3D
+    | MomentumArray4D,
+    mask_identity: bool,
+) -> ScalarCollection:
+    is_nonzero = array.rho2 != 0
+    if isinstance(array, Spatial):
+        is_nonzero = numpy.logical_or(is_nonzero, array.z != 0)
+    if isinstance(array, Lorentz):
+        is_nonzero = numpy.logical_or(is_nonzero, array.t2 != 0)
+
+    return ak.count_nonzero(is_nonzero, axis=1)
+
+
+for reducer, impl in (
+    (ak.sum, _reduce_sum),
+    (ak.count, _reduce_count),
+    (ak.count_nonzero, _reduce_count_nonzero),
+):
+    for dim in range(2, 5):
+        behavior[reducer, f"Vector{dim}D"] = impl
+        behavior[reducer, f"Momentum{dim}D"] = impl
