@@ -1,4 +1,4 @@
-# Copyright (c) 2019-2021, Jonas Eschle, Jim Pivarski, Eduardo Rodrigues, and Henry Schreiner.
+# Copyright (c) 2019-2024, Jonas Eschle, Jim Pivarski, Eduardo Rodrigues, and Henry Schreiner.
 #
 # Distributed under the 3-clause BSD license, see accompanying file LICENSE
 # or https://github.com/scikit-hep/vector for details.
@@ -32,6 +32,7 @@ from vector._methods import (
     LongitudinalZ,
     Lorentz,
     LorentzMomentum,
+    Momentum,
     Planar,
     PlanarMomentum,
     Spatial,
@@ -52,10 +53,65 @@ from vector._methods import (
     _repr_momentum_to_generic,
     _ttype,
 )
-from vector._typeutils import FloatArray, ScalarCollection
+from vector._typeutils import BoolCollection, FloatArray, ScalarCollection
 
 ArrayLike = ScalarCollection
 
+T = typing.TypeVar("T", bound="VectorNumpy")
+V = typing.TypeVar("V")
+
+SameVectorNumpyType = typing.TypeVar("SameVectorNumpyType", bound="VectorNumpy")
+
+
+def _reduce_sum(
+    a: T,
+    axis: int | None = None,
+    dtype: typing.Any = None,
+    out: typing.Any = None,
+    keepdims: bool = False,
+    initial: typing.Any = None,
+    where: typing.Any = None,
+) -> T:
+    if where is not None:
+        raise ValueError("cannot invoke reducer with `where` argument")
+    if initial is not None:
+        raise ValueError("cannot invoke reducer with `initial` argument")
+    if out is not None:
+        raise ValueError("cannot invoke reducer with `out` argument")
+    if dtype is not None:
+        raise ValueError("cannot invoke reducer with `dtype` argument")
+
+    fields: dict[str, typing.Any] = {}
+    if isinstance(a, Lorentz):
+        fields["E"] = numpy.sum(a.t, axis=axis, keepdims=keepdims)
+    if isinstance(a, Spatial):
+        fields["pz"] = numpy.sum(a.z, axis=axis, keepdims=keepdims)
+
+    assert isinstance(a, Planar)
+    fields["px"] = numpy.sum(a.x, axis=axis, keepdims=keepdims)
+    fields["py"] = numpy.sum(a.y, axis=axis, keepdims=keepdims)
+
+    # Convert between representations
+    if not isinstance(a, Momentum):
+        fields = {_repr_momentum_to_generic[n]: v for n, v in fields.items()}
+
+    return array(fields)
+
+
+def _reduce_count_nonzero(
+    a: T, axis: int | None = None, *, keepdims: bool = False
+) -> ScalarCollection:
+    if isinstance(a, Planar):
+        is_nonzero = a.rho2 != 0
+    else:
+        raise AssertionError
+    if isinstance(a, Spatial):
+        is_nonzero = numpy.logical_or(is_nonzero, a.z != 0)
+    if isinstance(a, Lorentz):
+        is_nonzero = numpy.logical_or(is_nonzero, a.t2 != 0)
+
+    return numpy.count_nonzero(is_nonzero, axis=axis, keepdims=keepdims)
+
 
 def _array_from_columns(columns: dict[str, ArrayLike]) -> ArrayLike:
     """
@@ -95,13 +151,13 @@ def _array_from_columns(columns: dict[str, ArrayLike]) -> ArrayLike:
         elif isinstance(columns[x], collections.abc.Sized):
             thisshape = (len(columns[x]),)
         else:
-            raise TypeError(f"column {repr(x)} has no length")
+            raise TypeError(f"column {x!r} has no length")
 
         dtype.append(thisdtype)
         if shape is None:
             shape = thisshape
         elif shape != thisshape:
-            raise ValueError(f"column {repr(x)} has a different shape than the others")
+            raise ValueError(f"column {x!r} has a different shape than the others")
 
     assert shape is not None
     array = numpy.empty(shape, dtype)
@@ -166,11 +222,15 @@ def _getitem(
                 *(out[x] for x in _coordinate_class_to_names[_ttype(array)])  # type: ignore[arg-type]
             )
         if temporal is not None:
-            return array.ObjectClass(azimuthal, longitudinal, temporal)  # type: ignore[call-arg, arg-type, return-value]
+            return array.ObjectClass(
+                azimuthal=azimuthal,
+                longitudinal=longitudinal,  # type: ignore[arg-type, return-value]
+                temporal=temporal,  # type: ignore[arg-type]
+            )
         elif longitudinal is not None:
-            return array.ObjectClass(azimuthal, longitudinal)  # type: ignore[call-arg, arg-type, return-value]
+            return array.ObjectClass(azimuthal=azimuthal, longitudinal=longitudinal)  # type: ignore[arg-type, return-value]
         elif azimuthal is not None:
-            return array.ObjectClass(azimuthal)  # type: ignore[call-arg, return-value]
+            return array.ObjectClass(azimuthal=azimuthal)  # type: ignore[return-value]
         else:
             return array.ObjectClass(*out.view(numpy.ndarray))  # type: ignore[misc, return-value]
 
@@ -229,7 +289,7 @@ def _has(
 
 
 def _toarrays(
-    result: tuple[ScalarCollection, ...] | ScalarCollection
+    result: tuple[ScalarCollection, ...] | ScalarCollection,
 ) -> tuple[FloatArray, ...]:
     """
     Converts a tuple of values to a tuple of ``numpy.array``s.
@@ -308,7 +368,7 @@ def _is_type_safe(
     | MomentumNumpy4D
     | CoordinatesNumpy,
 ) -> bool:
-    for i in range(0, len(array.dtype)):  # type: ignore[arg-type]
+    for i in range(len(array.dtype)):  # type: ignore[arg-type]
         if not issubclass(
             array.dtype[i].type, (numpy.integer, numpy.floating)
         ) or issubclass(array.dtype[i].type, numpy.timedelta64):
@@ -321,12 +381,10 @@ class GetItem:
     _IS_MOMENTUM: typing.ClassVar[bool]
 
     @typing.overload
-    def __getitem__(self, where: str) -> FloatArray:
-        ...
+    def __getitem__(self, where: str) -> FloatArray: ...
 
     @typing.overload
-    def __getitem__(self, where: typing.Any) -> float | FloatArray:
-        ...
+    def __getitem__(self, where: typing.Any) -> float | FloatArray: ...
 
     def __getitem__(self, where: typing.Any) -> float | FloatArray:
         return _getitem(self, where, self.__class__._IS_MOMENTUM)  # type: ignore[arg-type]
@@ -366,7 +424,7 @@ class AzimuthalNumpyXY(AzimuthalNumpy, AzimuthalXY, GetItem, FloatArray):  # typ
         >>> import vector
         >>> vector.backends.numpy.AzimuthalNumpyXY([(1, 1), (2.1, 3.1)], dtype=[("x", float), ("y", float)])
         AzimuthalNumpyXY([(1. , 1. ), (2.1, 3.1)],
-                        dtype=[('x', '<f8'), ('y', '<f8')])
+                         dtype=[('x', '<f8'), ('y', '<f8')])
     """
 
     ObjectClass = vector.backends.object.AzimuthalObjectXY
@@ -375,7 +433,7 @@ class AzimuthalNumpyXY(AzimuthalNumpy, AzimuthalXY, GetItem, FloatArray):  # typ
     def __new__(cls, *args: typing.Any, **kwargs: typing.Any) -> AzimuthalNumpyXY:
         return numpy.array(*args, **kwargs).view(cls)
 
-    def __array_finalize__(self, obj: typing.Any) -> None:  # type: ignore[override]
+    def __array_finalize__(self, obj: typing.Any) -> None:
         if not _has(self, ("x", "y")):
             raise TypeError(
                 f"{type(self).__name__} must have a structured dtype containing "
@@ -417,7 +475,7 @@ class AzimuthalNumpyRhoPhi(AzimuthalNumpy, AzimuthalRhoPhi, GetItem, FloatArray)
         >>> import vector
         >>> vector.backends.numpy.AzimuthalNumpyRhoPhi([(1, 1), (2.1, 3.1)], dtype=[("rho", float), ("phi", float)])
         AzimuthalNumpyRhoPhi([(1. , 1. ), (2.1, 3.1)],
-                            dtype=[('rho', '<f8'), ('phi', '<f8')])
+                             dtype=[('rho', '<f8'), ('phi', '<f8')])
     """
 
     ObjectClass = vector.backends.object.AzimuthalObjectRhoPhi
@@ -426,7 +484,7 @@ class AzimuthalNumpyRhoPhi(AzimuthalNumpy, AzimuthalRhoPhi, GetItem, FloatArray)
     def __new__(cls, *args: typing.Any, **kwargs: typing.Any) -> AzimuthalNumpyRhoPhi:
         return numpy.array(*args, **kwargs).view(cls)
 
-    def __array_finalize__(self, obj: typing.Any) -> None:  # type: ignore[override]
+    def __array_finalize__(self, obj: typing.Any) -> None:
         if not _has(self, ("rho", "phi")):
             raise TypeError(
                 f"{type(self).__name__} must have a structured dtype containing "
@@ -476,7 +534,7 @@ class LongitudinalNumpyZ(LongitudinalNumpy, LongitudinalZ, GetItem, FloatArray):
     def __new__(cls, *args: typing.Any, **kwargs: typing.Any) -> LongitudinalNumpyZ:
         return numpy.array(*args, **kwargs).view(cls)
 
-    def __array_finalize__(self, obj: typing.Any) -> None:  # type: ignore[override]
+    def __array_finalize__(self, obj: typing.Any) -> None:
         if not _has(self, ("z",)):
             raise TypeError(
                 f"{type(self).__name__} must have a structured dtype containing "
@@ -521,7 +579,7 @@ class LongitudinalNumpyTheta(LongitudinalNumpy, LongitudinalTheta, GetItem, Floa
     def __new__(cls, *args: typing.Any, **kwargs: typing.Any) -> LongitudinalNumpyTheta:
         return numpy.array(*args, **kwargs).view(cls)
 
-    def __array_finalize__(self, obj: typing.Any) -> None:  # type: ignore[override]
+    def __array_finalize__(self, obj: typing.Any) -> None:
         if not _has(self, ("theta",)):
             raise TypeError(
                 f"{type(self).__name__} must have a structured dtype containing "
@@ -566,7 +624,7 @@ class LongitudinalNumpyEta(LongitudinalNumpy, LongitudinalEta, GetItem, FloatArr
     def __new__(cls, *args: typing.Any, **kwargs: typing.Any) -> LongitudinalNumpyEta:
         return numpy.array(*args, **kwargs).view(cls)
 
-    def __array_finalize__(self, obj: typing.Any) -> None:  # type: ignore[override]
+    def __array_finalize__(self, obj: typing.Any) -> None:
         if not _has(self, ("eta",)):
             raise TypeError(
                 f"{type(self).__name__} must have a structured dtype containing "
@@ -611,7 +669,7 @@ class TemporalNumpyT(TemporalNumpy, TemporalT, GetItem, FloatArray):  # type: ig
     def __new__(cls, *args: typing.Any, **kwargs: typing.Any) -> TemporalNumpyT:
         return numpy.array(*args, **kwargs).view(cls)
 
-    def __array_finalize__(self, obj: typing.Any) -> None:  # type: ignore[override]
+    def __array_finalize__(self, obj: typing.Any) -> None:
         if not _has(self, ("t",)):
             raise TypeError(
                 f"{type(self).__name__} must have a structured dtype containing "
@@ -648,7 +706,7 @@ class TemporalNumpyTau(TemporalNumpy, TemporalTau, GetItem, FloatArray):  # type
     def __new__(cls, *args: typing.Any, **kwargs: typing.Any) -> TemporalNumpyTau:
         return numpy.array(*args, **kwargs).view(cls)
 
-    def __array_finalize__(self, obj: typing.Any) -> None:  # type: ignore[override]
+    def __array_finalize__(self, obj: typing.Any) -> None:
         if not _has(self, ("tau",)):
             raise TypeError(
                 f"{type(self).__name__} must have a structured dtype containing "
@@ -677,7 +735,7 @@ class TemporalNumpyTau(TemporalNumpy, TemporalTau, GetItem, FloatArray):  # type
 
 
 class VectorNumpy(Vector, GetItem):
-    """One dimensional vector class for the NumPy backend."""
+    """Mixin class for NumPy vectors."""
 
     lib = numpy
     dtype: numpy.dtype[typing.Any]
@@ -688,15 +746,38 @@ class VectorNumpy(Vector, GetItem):
         rtol: float | FloatArray = 1e-05,
         atol: float | FloatArray = 1e-08,
         equal_nan: bool | FloatArray = False,
-    ) -> FloatArray:
+    ) -> BoolCollection:
         """Like ``np.ndarray.allclose``, but for VectorNumpy."""
         return self.isclose(other, rtol=rtol, atol=atol, equal_nan=equal_nan).all()
 
+    def sum(
+        self: SameVectorNumpyType,
+        axis: int | None = None,
+        dtype: numpy.dtype[typing.Any] | str | None = None,
+        out: ArrayLike | None = None,
+        keepdims: bool = False,
+        initial: typing.Any = None,
+        where: typing.Any = None,
+    ) -> SameVectorNumpyType:
+        return typing.cast(
+            SameVectorNumpyType,
+            # pylint: disable-next=unexpected-keyword-arg
+            numpy.sum(
+                self,
+                axis=axis,
+                dtype=dtype,
+                out=out,
+                keepdims=keepdims,
+                initial=initial,
+                where=where,
+            ),  # type: ignore[call-overload]
+        )
+
     def __eq__(self, other: typing.Any) -> typing.Any:
-        return numpy.equal(self, other)
+        return numpy.equal(self, other)  # type: ignore[call-overload]
 
     def __ne__(self, other: typing.Any) -> typing.Any:
-        return numpy.not_equal(self, other)
+        return numpy.not_equal(self, other)  # type: ignore[call-overload]
 
     def __reduce__(self) -> str | tuple[typing.Any, ...]:
         pickled_state = super().__reduce__()
@@ -716,7 +797,7 @@ class VectorNumpy(Vector, GetItem):
     ) -> typing.Any:
         """
         Implements NumPy's ``ufunc``s for ``VectorNumpy``. The current implementation
-        includes ``numpy.absolute``, ``numpy.add``, ``numpy.subtract``, ``numpy.multipy``,
+        includes ``numpy.absolute``, ``numpy.add``, ``numpy.subtract``, ``numpy.multiply``,
         ``numpy.positive``, ``numpy.negative``, ``numpy.true_divide``, ``numpy.power``,
         ``numpy.square``, ``numpy.sqrt``, ``numpy.cbrt``, ``numpy.matmul``, ``numpy.equal``,
         and ``numpy.not_equal``.
@@ -933,6 +1014,10 @@ class VectorNumpy(Vector, GetItem):
             return type(self).isclose(*args, **kwargs)
         elif func is numpy.allclose:
             return type(self).allclose(*args, **kwargs)
+        elif func is numpy.sum:
+            return _reduce_sum(*args, **kwargs)
+        elif func is numpy.count_nonzero:
+            return _reduce_count_nonzero(*args, **kwargs)
         else:
             return NotImplemented
 
@@ -949,7 +1034,7 @@ class VectorNumpy2D(VectorNumpy, Planar, Vector2D, FloatArray):  # type: ignore[
         ...               dtype=[('x', float), ('y', float)])
         >>> vec
         VectorNumpy2D([(1.1, 2.1), (1.2, 2.2), (1.3, 2.3), (1.4, 2.4), (1.5, 2.5)],
-                    dtype=[('x', '<f8'), ('y', '<f8')])
+                      dtype=[('x', '<f8'), ('y', '<f8')])
     """
 
     ObjectClass = vector.backends.object.VectorObject2D
@@ -964,7 +1049,7 @@ class VectorNumpy2D(VectorNumpy, Planar, Vector2D, FloatArray):  # type: ignore[
             array = numpy.array(*args, **kwargs)
         return array.view(cls)
 
-    def __array_finalize__(self, obj: typing.Any) -> None:  # type: ignore[override]
+    def __array_finalize__(self, obj: typing.Any) -> None:
         if obj is None:
             return
 
@@ -1001,7 +1086,7 @@ class VectorNumpy2D(VectorNumpy, Planar, Vector2D, FloatArray):  # type: ignore[
             ... ], dtype=[("x", float), ("y", float)])
             >>> vec.azimuthal
             AzimuthalNumpyXY([(1.1, 2.1), (1.2, 2.2), (1.3, 2.3), (1.4, 2.4),
-                            (1.5, 2.5)], dtype=[('x', '<f8'), ('y', '<f8')])
+                              (1.5, 2.5)], dtype=[('x', '<f8'), ('y', '<f8')])
         """
         return self.view(self._azimuthal_type)  # type: ignore[return-value]
 
@@ -1029,6 +1114,7 @@ class VectorNumpy2D(VectorNumpy, Planar, Vector2D, FloatArray):  # type: ignore[
 
         elif (
             len(returns) == 1
+            or (len(returns) == 2 and returns[1] is None)
             and isinstance(returns[0], type)
             and issubclass(returns[0], Azimuthal)
         ):
@@ -1044,24 +1130,7 @@ class VectorNumpy2D(VectorNumpy, Planar, Vector2D, FloatArray):  # type: ignore[
             return out.view(cls.ProjectionClass2D)
 
         elif (
-            len(returns) == 2
-            and isinstance(returns[0], type)
-            and issubclass(returns[0], Azimuthal)
-            and returns[1] is None
-        ):
-            result = _toarrays(result)
-            dtype = [
-                (name, result[i].dtype)
-                for i, name in enumerate(_coordinate_class_to_names[returns[0]])
-            ]
-
-            out = numpy.empty(_shape_of(result), dtype=dtype)
-            for i, name in enumerate(_coordinate_class_to_names[returns[0]]):
-                out[name] = result[i]
-            return out.view(cls.ProjectionClass2D)
-
-        elif (
-            len(returns) == 2
+            (len(returns) == 2 or (len(returns) == 3 and returns[2] is None))
             and isinstance(returns[0], type)
             and issubclass(returns[0], Azimuthal)
             and isinstance(returns[1], type)
@@ -1086,33 +1155,6 @@ class VectorNumpy2D(VectorNumpy, Planar, Vector2D, FloatArray):  # type: ignore[
                 i += 1
             return out.view(cls.ProjectionClass3D)
 
-        elif (
-            len(returns) == 3
-            and isinstance(returns[0], type)
-            and issubclass(returns[0], Azimuthal)
-            and isinstance(returns[1], type)
-            and issubclass(returns[1], Longitudinal)
-            and returns[2] is None
-        ):
-            result = _toarrays(result)
-            dtype = []
-            i = 0
-            for name in _coordinate_class_to_names[returns[0]]:
-                dtype.append((name, result[i].dtype))
-                i += 1
-            for name in _coordinate_class_to_names[returns[1]]:
-                dtype.append((name, result[i].dtype))
-                i += 1
-            out = numpy.empty(_shape_of(result), dtype=dtype)
-            i = 0
-            for name in _coordinate_class_to_names[returns[0]]:
-                out[name] = result[i]
-                i += 1
-            for name in _coordinate_class_to_names[returns[1]]:
-                out[name] = result[i]
-                i += 1
-            return out.view(cls.ProjectionClass3D)
-
         elif (
             len(returns) == 3
             and isinstance(returns[0], type)
@@ -1173,7 +1215,7 @@ class MomentumNumpy2D(PlanarMomentum, VectorNumpy2D):  # type: ignore[misc]
     _IS_MOMENTUM = True
     dtype: numpy.dtype[typing.Any]
 
-    def __array_finalize__(self, obj: typing.Any) -> None:  # type: ignore[override]
+    def __array_finalize__(self, obj: typing.Any) -> None:
         if obj is None:
             return
 
@@ -1215,16 +1257,18 @@ class VectorNumpy3D(VectorNumpy, Spatial, Vector3D, FloatArray):  # type: ignore
         ...               dtype=[('x', float), ('y', float), ('z', float)])
         >>> vec
         VectorNumpy3D([(1.1, 2.1, 3.1), (1.2, 2.2, 3.2), (1.3, 2.3, 3.3), (1.4, 2.4, 3.4),
-                    (1.5, 2.5, 3.5)], dtype=[('x', '<f8'), ('y', '<f8'), ('z', '<f8')])
+                       (1.5, 2.5, 3.5)], dtype=[('x', '<f8'), ('y', '<f8'), ('z', '<f8')])
     """
 
     ObjectClass = vector.backends.object.VectorObject3D
     _IS_MOMENTUM = False
 
     _azimuthal_type: type[AzimuthalNumpyXY] | type[AzimuthalNumpyRhoPhi]
-    _longitudinal_type: type[LongitudinalNumpyZ] | type[LongitudinalNumpyTheta] | type[
-        LongitudinalNumpyEta
-    ]
+    _longitudinal_type: (
+        type[LongitudinalNumpyZ]
+        | type[LongitudinalNumpyTheta]
+        | type[LongitudinalNumpyEta]
+    )
 
     def __new__(cls, *args: typing.Any, **kwargs: typing.Any) -> VectorNumpy3D:
         """Returns the object of ``VectorNumpy3D``. Behaves as ``__init__`` in this case."""
@@ -1234,7 +1278,7 @@ class VectorNumpy3D(VectorNumpy, Spatial, Vector3D, FloatArray):  # type: ignore
             array = numpy.array(*args, **kwargs)
         return array.view(cls)
 
-    def __array_finalize__(self, obj: typing.Any) -> None:  # type: ignore[override]
+    def __array_finalize__(self, obj: typing.Any) -> None:
         if obj is None:
             return
 
@@ -1343,6 +1387,7 @@ class VectorNumpy3D(VectorNumpy, Spatial, Vector3D, FloatArray):  # type: ignore
 
         elif (
             len(returns) == 2
+            or (len(returns) == 3 and returns[2] is None)
             and isinstance(returns[0], type)
             and issubclass(returns[0], Azimuthal)
             and isinstance(returns[1], type)
@@ -1367,33 +1412,6 @@ class VectorNumpy3D(VectorNumpy, Spatial, Vector3D, FloatArray):  # type: ignore
                 i += 1
             return out.view(cls.ProjectionClass3D)
 
-        elif (
-            len(returns) == 3
-            and isinstance(returns[0], type)
-            and issubclass(returns[0], Azimuthal)
-            and isinstance(returns[1], type)
-            and issubclass(returns[1], Longitudinal)
-            and returns[2] is None
-        ):
-            result = _toarrays(result)
-            dtype = []
-            i = 0
-            for name in _coordinate_class_to_names[returns[0]]:
-                dtype.append((name, result[i].dtype))
-                i += 1
-            for name in _coordinate_class_to_names[returns[1]]:
-                dtype.append((name, result[i].dtype))
-                i += 1
-            out = numpy.empty(_shape_of(result), dtype=dtype)
-            i = 0
-            for name in _coordinate_class_to_names[returns[0]]:
-                out[name] = result[i]
-                i += 1
-            for name in _coordinate_class_to_names[returns[1]]:
-                out[name] = result[i]
-                i += 1
-            return out.view(cls.ProjectionClass3D)
-
         elif (
             len(returns) == 3
             and isinstance(returns[0], type)
@@ -1447,14 +1465,14 @@ class MomentumNumpy3D(SpatialMomentum, VectorNumpy3D):  # type: ignore[misc]
         ...               dtype=[('px', float), ('py', float), ('pz', float)])
         >>> vec
         MomentumNumpy3D([(1.1, 2.1, 3.1), (1.2, 2.2, 3.2), (1.3, 2.3, 3.3), (1.4, 2.4, 3.4),
-                        (1.5, 2.5, 3.5)], dtype=[('x', '<f8'), ('y', '<f8'), ('z', '<f8')])
+                         (1.5, 2.5, 3.5)], dtype=[('x', '<f8'), ('y', '<f8'), ('z', '<f8')])
     """
 
     ObjectClass = vector.backends.object.MomentumObject3D
     _IS_MOMENTUM = True
     dtype: numpy.dtype[typing.Any]
 
-    def __array_finalize__(self, obj: typing.Any) -> None:  # type: ignore[override]
+    def __array_finalize__(self, obj: typing.Any) -> None:
         if obj is None:
             return
 
@@ -1506,17 +1524,19 @@ class VectorNumpy4D(VectorNumpy, Lorentz, Vector4D, FloatArray):  # type: ignore
         ...               dtype=[('x', float), ('y', float), ('z', float), ('t', float)])
         >>> vec
         VectorNumpy4D([(1.1, 2.1, 3.1, 4.1), (1.2, 2.2, 3.2, 4.2), (1.3, 2.3, 3.3, 4.3),
-                    (1.4, 2.4, 3.4, 4.4), (1.5, 2.5, 3.5, 4.5)],
-                    dtype=[('x', '<f8'), ('y', '<f8'), ('z', '<f8'), ('t', '<f8')])
+                       (1.4, 2.4, 3.4, 4.4), (1.5, 2.5, 3.5, 4.5)],
+                      dtype=[('x', '<f8'), ('y', '<f8'), ('z', '<f8'), ('t', '<f8')])
     """
 
     ObjectClass = vector.backends.object.VectorObject4D
     _IS_MOMENTUM = False
 
     _azimuthal_type: type[AzimuthalNumpyXY] | type[AzimuthalNumpyRhoPhi]
-    _longitudinal_type: type[LongitudinalNumpyZ] | type[LongitudinalNumpyTheta] | type[
-        LongitudinalNumpyEta
-    ]
+    _longitudinal_type: (
+        type[LongitudinalNumpyZ]
+        | type[LongitudinalNumpyTheta]
+        | type[LongitudinalNumpyEta]
+    )
     _temporal_type: type[TemporalNumpyT] | type[TemporalNumpyTau]
 
     def __new__(cls, *args: typing.Any, **kwargs: typing.Any) -> VectorNumpy4D:
@@ -1527,7 +1547,7 @@ class VectorNumpy4D(VectorNumpy, Lorentz, Vector4D, FloatArray):  # type: ignore
             array = numpy.array(*args, **kwargs)
         return array.view(cls)
 
-    def __array_finalize__(self, obj: typing.Any) -> None:  # type: ignore[override]
+    def __array_finalize__(self, obj: typing.Any) -> None:
         if obj is None:
             return
 
@@ -1765,7 +1785,7 @@ class MomentumNumpy4D(LorentzMomentum, VectorNumpy4D):  # type: ignore[misc]
         ...               dtype=[('px', float), ('py', float), ('pz', float), ('t', float)])
         >>> vec
         MomentumNumpy4D([(1.1, 2.1, 3.1, 4.1), (1.2, 2.2, 3.2, 4.2), (1.3, 2.3, 3.3, 4.3),
-                        (1.4, 2.4, 3.4, 4.4), (1.5, 2.5, 3.5, 4.5)],
+                         (1.4, 2.4, 3.4, 4.4), (1.5, 2.5, 3.5, 4.5)],
                         dtype=[('x', '<f8'), ('y', '<f8'), ('z', '<f8'), ('t', '<f8')])
     """
 
@@ -1773,7 +1793,7 @@ class MomentumNumpy4D(LorentzMomentum, VectorNumpy4D):  # type: ignore[misc]
     _IS_MOMENTUM = True
     dtype: numpy.dtype[typing.Any]
 
-    def __array_finalize__(self, obj: typing.Any) -> None:  # type: ignore[override]
+    def __array_finalize__(self, obj: typing.Any) -> None:
         if obj is None:
             return
 
@@ -1906,28 +1926,34 @@ VectorNumpy2D.ProjectionClass2D = VectorNumpy2D
 VectorNumpy2D.ProjectionClass3D = VectorNumpy3D
 VectorNumpy2D.ProjectionClass4D = VectorNumpy4D
 VectorNumpy2D.GenericClass = VectorNumpy2D
+VectorNumpy2D.MomentumClass = MomentumNumpy2D
 
 MomentumNumpy2D.ProjectionClass2D = MomentumNumpy2D
 MomentumNumpy2D.ProjectionClass3D = MomentumNumpy3D
 MomentumNumpy2D.ProjectionClass4D = MomentumNumpy4D
 MomentumNumpy2D.GenericClass = VectorNumpy2D
+MomentumNumpy2D.MomentumClass = MomentumNumpy2D
 
 VectorNumpy3D.ProjectionClass2D = VectorNumpy2D
 VectorNumpy3D.ProjectionClass3D = VectorNumpy3D
 VectorNumpy3D.ProjectionClass4D = VectorNumpy4D
 VectorNumpy3D.GenericClass = VectorNumpy3D
+VectorNumpy3D.MomentumClass = MomentumNumpy3D
 
 MomentumNumpy3D.ProjectionClass2D = MomentumNumpy2D
 MomentumNumpy3D.ProjectionClass3D = MomentumNumpy3D
 MomentumNumpy3D.ProjectionClass4D = MomentumNumpy4D
 MomentumNumpy3D.GenericClass = VectorNumpy3D
+MomentumNumpy3D.MomentumClass = MomentumNumpy3D
 
 VectorNumpy4D.ProjectionClass2D = VectorNumpy2D
 VectorNumpy4D.ProjectionClass3D = VectorNumpy3D
 VectorNumpy4D.ProjectionClass4D = VectorNumpy4D
 VectorNumpy4D.GenericClass = VectorNumpy4D
+VectorNumpy4D.MomentumClass = MomentumNumpy4D
 
 MomentumNumpy4D.ProjectionClass2D = MomentumNumpy2D
 MomentumNumpy4D.ProjectionClass3D = MomentumNumpy3D
 MomentumNumpy4D.ProjectionClass4D = MomentumNumpy4D
 MomentumNumpy4D.GenericClass = VectorNumpy4D
+MomentumNumpy4D.MomentumClass = MomentumNumpy4D
